#!/bin/bash

ver="1.6.1"

if [[ "$1" == "/help" ]]
then
	reset
	echo "======================================="
	echo "|  Perimeter 81 Log Collector v.$ver |"
	echo "======================================="
	echo ""
	echo "Regular Use:"
	echo "----------------------------"
	echo "sudo bash ./Customer_logs-v$ver.sh == Regular use with no attributes and flags (script must be run under sudo)"
	echo ""
	echo "Supported pre-populated attributes and flags:"
	echo "----------------------------"
	echo "sudo bash ./Customer_logs-v$ver.sh <TENANT_NAME> <USER_EMAIL> <TICKET_NUMBER> <CLEAN_FILES>"
	echo ""
	echo "   <TENANT_NAME>   - Mandatory - The workspace to which you're connecting (<TENANT_NAME>.perimeter81.com)"
	echo "   <USER_EMAIL>    - Mandatory - Your email address (preferably the one that you use to login to the platform)"
	echo "   <TICKET_NUMBER> - Optional  - The support ticket number for the issue (if you got it from our team). If set to 0 a new ticket will be created."
	echo "   <CLEAN_FILES>   - Optional  - (Y/n) - Delete generated files from the local machine once finished. Field is case-sensetive"
	echo ""
	echo "Example:"
	echo "========"
	echo "sudo bash ./Customer_logs-v$ver.sh internal user@email.com 0 Y"
	echo ""
	exit 1
fi

currentsha=$(shasum -a256 Customer_logs-v$ver.sh)

echo "0ab793eaa80ab837becbed8c5c5eb7278689d9a9 Customer_logs-v$ver.sh" | shasum -c
if [ $? != 0 ]; then
  echo 'Checksum failed - (BETA)'
fi

if [[ $EUID -ne 0 ]]; then
	echo "This script must be running as root. Please enter password to run as sudo."
   	sudo bash Customer_logs-v$ver.sh $1 $2 $3 $4
   	exit 1
fi

timestamp=$(date '+%s');
hostname=$(hostname -s)

reset
echo ""
echo ...............PERIMETER 81...............
echo ".                                        ."
echo ".  MMMMMMMMMMMMMMWXOkOKWMMMMMMMMMMMMMMM  ." =======================================
echo ".  MMMMMMMMMMMN0koc:::cox0NWMMMMMMMMMMM  ." \|  Perimeter 81 Log Collector v.$ver  \|
echo ".  MMMMMMMWXOxlc:cccccc:::ldOKWMMMMMMMM  ." =======================================
echo ".  MMMMNKkoc::cccccclccccc:::cox0NMMMMM  ." This script collects logs from $hostname
echo ".  MW0xlc::ccccclok0K0kdlc:cc::::ld0WMM  ." and sends them to Perimeter 81 Support.
echo ".  MKl:ccccccldOKXNNNNNNKkdlc::::::c0MM  ." All required information will be collected
echo ".  M0l:cc:cdOXNNNNNNNNNNNNNXOdc::::cOMM  ." and password protected.
echo ".  M0l:cc:lOKKXNNNNNNNNNNNXK0kc::::cOMM  ." By running the script you consent providing
echo ".  M0l:c::ckK00KKXNNNNXK0OOkkxc::::cOMM  ." the collected data to the Perimeter 81 team
echo ".  M0l::::ckK000000KK0OOkkkkkxc::::cOMM  ." for troubleshooting and tracking purposes.
echo ".  M0l::::ckK0K000000OOOkkkkkdc::::cOMM  ." 
echo ".  M0c::::cOKKKK00000Okkkkkxkdc:::::OMM  ." Please don\'t close the terminal until the 
echo ".  M0c:::::lxO0KK0000kkkkkxdoc::::::OMM  ." script provides a confirmation.
echo ".  M0c::::::::ldk0K0Okkxdlc::::::::c0MM  ." 
echo ".  M0c:::::::::::coxxoc::::::::::lxKWMM  ." For any issues with the script please
echo ".  M0c:::::::::::::::::::::::cdkKNMMMMM  ." contact support@perimeter81.com.
echo ".  M0c::::ldkxo:::::::::::lxOXWMMMMMMMM  ." 
echo ".  M0c:ok0NWMMNKkdc:::cokKNMMMMMMMMMMMM  ." Best Regards,
echo ".  MN0KWMMMMMMMMMWX0O0XWMMMMMMMMMMMMMMM  ." 
echo ".                                        ." The Perimeter 81 team
echo ...........support@perimeter81.com........
echo ""

spin()
{
  spinner="/|\\-/|\\-"
  while :
  do
    for i in `seq 0 7`
    do
      echo -n "${spinner:$i:1}"
      echo -en "\010"
      sleep 0.5
    done
  done
}

sleep 2
echo "Log Files will be coplied to a temporary folder (./$timestamp-$hostname/)"
echo ""
echo "Collecting Perimeter 81 Logs from $hostname..."
cp -R /var/log/Perimeter81 $timestamp-$hostname
mkdir $timestamp-$hostname/tmp
cp /tmp/Perimeter81.log $timestamp-$hostname/tmp/
mkdir $timestamp-$hostname/Library
cp ~/Library/Logs/Perimeter81.log $timestamp-$hostname/Library/
chmod -R 777 $timestamp-$hostname/Library
wg > $timestamp-$hostname/wg_status.log
echo "" >> $timestamp-$hostname/wg_status.log
cat /usr/local/var/wg/wgconf0.conf | grep Address >> $timestamp-$hostname/wg_status.log
cat /usr/local/var/wg/wgconf0.conf | grep DNS >> $timestamp-$hostname/wg_status.log
DNS=$(cat /usr/local/var/wg/wgconf0.conf | grep DNS)
DNS=$(echo "$DNS" | grep -o '[0-9]\+[.][0-9]\+[.][0-9]\+[.][0-9]\+')
EXT=$(cat /usr/local/var/wg/wgconf0.conf | grep Endpoint)
EXT=$(echo "$EXT" | grep -o '[0-9]\+[.][0-9]\+[.][0-9]\+[.][0-9]\+')
cat /usr/local/var/wg/wgconf0.conf | grep AllowedIPs >> $timestamp-$hostname/wg_status.log
cat /usr/local/var/wg/wgconf0.conf | grep Endpoint >> $timestamp-$hostname/wg_status.log
echo ""

if [[ "$4" == "Y" ]] || [[ "$4" == "n" ]]
then
	DELFILES="$4"
fi

if [[ -z "$3" ]]
then
	echo " || Please provide your support ticket number (Optional - you can keep it blank):"
	read ticket
else
	if [[ $3 -gt 99999 ]] || [[ $3 -lt 10000 ]]
	then
		if [[ $3 -eq "0" ]]
		then
			ticket=0
		else
			echo "Invalid ticket number."
			exit 1
		fi
	else
		ticket="$3"
	fi
fi

echo ""

if [ -z "$ticket" ]
then
		ticket=0
else
		echo ""
		echo "==================================================="
		echo " Recorded support reference $ticket"
		echo "==================================================="
		echo ""
fi

if [ -z "$1" ]
then
	echo " || Please provide your tenant name (<TENANT>.perimeter81.com):"
	read tenant
else
	tenant="$1"
fi

if [ -z "$tenant" ]
then
		echo "Tenant can't be empty"
		exit 1
else
		echo ""
		echo "==================================================="
		echo " Recorded tenant $tenant.perimeter81.com"
		echo "==================================================="
		echo ""
fi
regex="^(([A-Za-z0-9]+((\.|\-|\_|\+)?[A-Za-z0-9]?)*[A-Za-z0-9]+)|[A-Za-z0-9]+)@(([A-Za-z0-9]+)+((\.|\-|\_)?([A-Za-z0-9]+)+)*)+\.([A-Za-z]{2,})+$"
if [[ -z "$2" ]] || ! [[ "$2" =~ $regex ]]
then
	echo " || Please provide a valid Perimeter 81 Username (registered email address)?"
	read email
else
	email="$2"
fi
if [[ -z "$email" ]] || ! [[ "$email" =~ $regex ]]
then
		echo "Username has to be a valid email address"
		exit 1
else
		echo ""
		echo "=========================================="
		echo " Recorded username $email"
		echo "=========================================="
		echo ""
fi

echo "Collecting Network and Device information..."
spin &
SPIN_PID=$!
trap "kill -9 $SPIN_PID" `seq 0 15`
ifconfig > $timestamp-$hostname/ifconfig.log
traceroute -I perimeter81.com > $timestamp-$hostname/traceroute.log
netstat -rln -f inet > $timestamp-$hostname/routing-tables.log
lsof -i > $timestamp-$hostname/lsof.log
echo "Finished."
echo ""
dig sdp.pzero.perimeter81.com +trace > $timestamp-$hostname/dig-SDP.log
dig sdp.pzero.perimeter81.com @8.8.8.8 +trace >> $timestamp-$hostname/dig-SDP.log
dig dig sdp.pzero.perimeter81.com +short >> $timestamp-$hostname/dig-SDP.log
nc -vz sdp.pzero.perimeter81.com 5050 > $timestamp-$hostname/port5050-result.log

if [ -z "$DNS" ]
then
	echo "Running DIG to $tenant.perimeter81.com..."
	dig $tenant.perimeter81.com +trace > $timestamp-$hostname/dig.log
	dig $tenant.perimeter81.com @8.8.8.8 +trace >> $timestamp-$hostname/dig.log
	echo "Finished."
	echo ""
else
	echo "Running DIG to $DNS..."
	dig $DNS +trace > $timestamp-$hostname/dig.log
	dig $DNS @8.8.8.8 +trace >> $timestamp-$hostname/dig.log
	echo "Finished."
	echo ""
fi

kill -9 $SPIN_PID
echo "Running 6 seconds of PING to www.perimeter81.com..."
spin &
SPIN_PID=$!
trap "kill -9 $SPIN_PID" `seq 0 6`
ping www.perimeter81.com -t6 > $timestamp-$hostname/ping-perimeter81.com.log
echo "Finished."
kill -9 $SPIN_PID

if [ -z "$DNS" ]
then
	echo "Running 6 seconds of PING to $tenant.perimeter81.com..."
	spin &
	SPIN_PID=$!
	trap "kill -9 $SPIN_PID" `seq 0 6`
	ping $tenant.perimeter81.com -t15 > $timestamp-$hostname/ping-$tenant.log
	echo "Finished."
	kill -9 $SPIN_PID
else
	echo "Running 15 seconds of PING to Perimeter 81 Internal DNS IP ($DNS)..."
	spin &
	SPIN_PID=$!
	trap "kill -9 $SPIN_PID" `seq 0 6`
	ping $DNS -t15 > $timestamp-$hostname/ping-$DNS.log
	echo "Finished."
	kill -9 $SPIN_PID
fi

echo ""
echo ""
echo "Compressing all collected files..."
zip -P $timestamp -r LogFiles-$timestamp-$hostname.zip $timestamp-$hostname -s 3m
echo ""
filecount=$(ls LogFiles-$timestamp-$hostname.z* | wc -l)
echo "Deleting temp foleder (./$timestamp-$hostname)..."
rm -R $timestamp-$hostname
echo ""
echo "Posting LogFiles-$timestamp-$hostname.zip in ${filecount//[[:blank:]]/} parts from tenant:$tenant for ticket:$ticket -- user:$email machine:$hostname to the Perimeter 81 Log Collector Bucket"
curl --request POST -H "Content-Type: application/zip" --data-binary @LogFiles-$timestamp-$hostname.zip https://ksr7tdm0ij.execute-api.us-east-2.amazonaws.com/v1/uploadmaclogs > response.log
filepath=$(cat response.log | cut -d'!' -f 2)
payload="{\"email\":\"$email\", \"ticket\":$ticket, \"tenant\":\"$tenant\", \"filepath\":\"$filepath\", \"filename\":\"LogFiles-$timestamp-$hostname.zip\"}"
echo "$payload"
echo "" >> response.log
curl -d "$payload" -H "Content-Type: application/json" -X POST https://ksr7tdm0ij.execute-api.us-east-2.amazonaws.com/v1/ > response.log
echo ""
cat response.log
ticket=$(cat response.log | cut -d'#' -f 2)
ticket=${ticket:0:5}

for (( i=1; i<${filecount//[[:blank:]]/}; i++ ))
do
	if [ $i -lt 10 ]
	then
		curl --request POST -H "Content-Type: application/zip" --data-binary @LogFiles-$timestamp-$hostname.z0$i https://ksr7tdm0ij.execute-api.us-east-2.amazonaws.com/v1/uploadmaclogs > response.log
		filepath=$(cat response.log | cut -d'!' -f 2)
		payload="{\"email\":\"$email\", \"ticket\":$ticket, \"tenant\":\"$tenant\", \"filepath\":\"$filepath\", \"filename\":\"LogFiles-$timestamp-$hostname.z0$i\", \"part\":\"$i\"}"
	else
		curl --request POST -H "Content-Type: application/zip" --data-binary @LogFiles-$timestamp-$hostname.z$i https://ksr7tdm0ij.execute-api.us-east-2.amazonaws.com/v1/uploadmaclogs > response.log
		filepath=$(cat response.log | cut -d'!' -f 2)
		payload="{\"email\":\"$email\", \"ticket\":$ticket, \"tenant\":\"$tenant\", \"filepath\":\"$filepath\", \"filename\":\"LogFiles-$timestamp-$hostname.z$i\", \"part\":\"$i\"}"
	fi
	echo "$payload"
	echo "" >> response.log
	curl -d "$payload" -H "Content-Type: application/json" -X POST https://ksr7tdm0ij.execute-api.us-east-2.amazonaws.com/v1/ > response.log
	echo ""
	cat response.log
done

ticket=$(cat response.log | cut -d'#' -f 2)
ticket=${ticket:0:5}
kill -9 $SPIN_PID
echo ""
if [ -z "$DELFILES" ]
then
	echo " Should the collected files be deleted (Y/n):"
	read DELFILES
fi

if ! [ $DELFILES == "n" ]
then
	echo "Cleaning up..."
	rm LogFiles-$timestamp-$hostname.z*
fi

echo ""
echo "Thank you for your help!"
echo "Perimeter 81 Support (support@perimeter81.com)"
